<?xml version="1.0" encoding="UTF-8"?>
<!--
     XLIFF Version 2.1
     Committee Specification Draft 02 / Public Review Draft 02
     07 February 2017
     Copyright (c) OASIS Open 2017. All Rights Reserved.
     Source: http://docs.oasis-open.org/xliff/xliff-core/v2.1/csprd02/schemas/
     Latest version of narrative specification: http://docs.oasis-open.org/xliff/xliff-core/v2.1/xliff-core-v2.1.html
     TC IPR Statement: https://www.oasis-open.org/committees/xliff/ipr.php
     
-->
<!DOCTYPE schematron [
<!-- Entities for XLIFF V2.x publishing.................................... -->
<!--copy all of these to all *.sch files and also to the Oxygen framework for validating Docbook 4.5 if you use Oxygen -->
<!ENTITY name "xliff-core-v2.1">
<!ENTITY pversion "2.0">
<!ENTITY version "2.1">
<!ENTITY bschversion "2.0">
<!ENTITY cschversion "2.1">

<!ENTITY stage "csprd02">
<!ENTITY pstage "csprd01">
<!ENTITY standard "Commitee Specification Draft 02 / Public Review Draft 02">

<!ENTITY this-loc "http://docs.oasis-open.org/xliff/xliff-core/v&version;/&stage;">
<!ENTITY previous-loc "http://docs.oasis-open.org/xliff/xliff-core/v&version;/&pstage;">
<!ENTITY latest-loc "http://docs.oasis-open.org/xliff/xliff-core/v&version;">
<!ENTITY pubdate "07 February &pubyear;">
<!ENTITY pubyear "2017">
<!ENTITY releaseinfo "Standards Track Work Product">
<!-- End of XLIFF V2.x publishing entities -->
]>


<sch:schema xmlns:sch="http://purl.oclc.org/dsdl/schematron" queryBinding='xslt2'>
    <sch:title>Schematron rules for checking the constraints of the Change Tracking module against the XLIFF &version; spec</sch:title>
    <sch:ns prefix="ctr" uri="urn:oasis:names:tc:xliff:changetracking:2.1"/>
    <sch:ns prefix="xlf" uri="urn:oasis:names:tc:xliff:document:2.0"/>
    <sch:pattern id="K5CTR">
        <sch:title>The value [of id] must be unique among all 'data' id attribute values within the enclosing 'originalData' element</sch:title>
        <sch:rule context="xlf:data[ancestor::ctr:changeTrack]">
            <sch:let name="id" value="current()/@id"/>
            <sch:let name="fragid" value="concat('f=',ancestor::xlf:file/@id)"/>
            <sch:report diagnostics="fragid-report" see="&this-loc;/&name;-&stage;.html#id"
                test="following-sibling::xlf:data[@id=$id]">
                id duplication found. The value '<sch:value-of select="$id"/>' is used more than once for 'data' elements 
                within the enclosing 'originalData' in the CTR module.
            </sch:report>
        </sch:rule>
    </sch:pattern>
    <sch:pattern id="K6CTR-S">
        <sch:title>Except for the above exception [target duplication], the value [of id attribute] must be unique among all of the above 
            [segment, ignorable, mkr, sm,pc, sc, ec, ph] within the enclosing 'revision' element</sch:title>
        <sch:rule context="xlf:source[ancestor::ctr:revision]//xlf:*[@id]|
            xlf:segment[@id][ancestor::ctr:revision]| xlf:ignorable[@id][ancestor::ctr:revision]">
            <sch:let name="id" value="current()/@id"/>
            <sch:report see="&this-loc;/&name;-&stage;.html#id"
                test="count(ancestor::ctr:revision//xlf:*[@id=$id][ancestor-or-self::xlf:segment| ancestor-or-self::xlf:ignorable][not(ancestor::xlf:target)])>1">
                id duplication found. The value '<sch:value-of select="$id"/>' is used more then once among inline and/or 'segmen'/'ignorable' 
                elements within the enclosing 'revision'.
            </sch:report>
        </sch:rule>
    </sch:pattern>
    <sch:pattern id="K6CTR-T">
        <sch:title>The inline elements enclosed by a 'target' element must use the duplicate id values of their corresponding inline elements enclosed within the sibling 'source' element if and only if those corresponding elements exist</sch:title>
        <sch:rule context="xlf:target[ancestor::ctr:revision]//xlf:*[@id]">
            <sch:let name="id" value="@id"/>
            <sch:let name="file-id" value="ancestor::xlf:file/@id"/>
            <sch:let name="unit-id" value="ancestor::xlf:unit/@id"/>
            <sch:let name="counter" value="count(ancestor::ctr:revision//xlf:*[@id=$id]
                [ancestor-or-self::xlf:segment| ancestor-or-self::xlf:ignorable])"/>
            <!-- If the counter is 1, then the id is unique and therefore valid -->
            <!-- If the counter is 2 (duplication exsists), the duplication must be 
            in the sibling source, have the same name and contain same attributes-->
            <!-- If the counter is more than 2 or the duplication is not in the right 
            position, then an error will be reported -->
            <sch:assert see="&this-loc;/&name;-&stage;.html#id"
                test="($counter=1) or ($counter=2 and
                count(ancestor::ctr:revision//xlf:source//xlf:*[@id=$id]
                [local-name()= local-name(current())])=1)">
                Invalid id used for element '<sch:name/>'. It must duplicate the 
                id of its corresponding element, enclosed within the 'source' 
                element or be unique in the scope of 'revision'.
            </sch:assert>
        </sch:rule>
    </sch:pattern>
    <sch:pattern id="ctr-F15">
        <sch:title>dataRef attribute must point to a data element within the same ctr:revision</sch:title>
        <sch:rule context="xlf:*[@dataRef][ancestor::ctr:revision]">
            <sch:assert test="ancestor::ctr:revision/xlf:originalData/xlf:data[@id=current()/@dataRef]">
                'dataRef' attribute must point to a 'data' element within the same 'ctr:revision' (allowed referencing pattern: dataRef="data-id").
            </sch:assert>
        </sch:rule>
    </sch:pattern>
    <sch:pattern id="ctr-F16">
        <sch:title>dataRefEnd attribute must point to a data element within the same ctr:revision</sch:title>
        <sch:rule context="xlf:pc[@dataRefEnd][ancestor::ctr:revision]">
            <sch:assert test="ancestor::ctr:revision/xlf:originalData/xlf:data[@id=current()/@dataRefEnd]">
                'dataRefEnd' attribute must point to a 'data' element within the same 'ctr:revision' (allowed referencing pattern: dataRefEnd="data-id").
            </sch:assert>
            <sch:assert test="@dataRefStart">
                'dataRefEnd' and 'dataRefStart' must be used in pair.
            </sch:assert>
        </sch:rule>
    </sch:pattern>
    <sch:pattern id="ctr-F17">
        <sch:title>dataRefStart attribute must point to a data element within the same ctr:revision</sch:title>
        <sch:rule context="xlf:pc[@dataRefStart][ancestor::ctr:revision]">
            <sch:assert test="ancestor::ctr:revision/xlf:originalData/xlf:data[@id=current()/@dataRefStart]">
                'dataRefStart' attribute must point to a 'data' element within the same 'ctr:revision' (allowed referencing pattern: dataRefStart="data-id").
            </sch:assert>
            <sch:assert test="@dataRefEnd">
                'dataRefStart' and 'dataRefEnd' must be used in pair.
            </sch:assert>
        </sch:rule>
    </sch:pattern>
    <sch:pattern id="ctr1">
        <sch:rule context="ctr:*[@id]">
            <sch:let name="fragid" value="concat('f=', ancestor::xlf:file/@id)"/>
            <sch:report test="count(ancestor::ctr:changeTrack//ctr:*[@id=current()/@id]>1)">
                The value of 'id' must be unique among all 'revisions', 'revision', 'item', 'unitItem' and 'contentItem' elements within the CTR module.
            </sch:report>
        </sch:rule>
    </sch:pattern>
    <sch:pattern id="ctr2">
        <sch:rule context="ctr:revisions[@appliesTo='source'] | ctr:revisions[@appliesTo='target']">
            <!-- source-counter represents the number of source elements within the same unit, which are children of segment or ignorable -->
            <sch:let name="source-counter" value="count(ancestor::xlf:unit//xlf:source[parent::xlf:*])"/>
            <!-- An error will be raised if more than one source is present in the unit and @ref is missing-->
            <sch:report test="$source-counter>1 and not(@ref)">
                The 'appliesTo' attribute is set to 'source' or 'target' and the 'unit' ancestor contains more than one 'source'/'target', but the 'ref' attribute is missing in &lt;revisions&gt;.
            </sch:report>
            <!-- An error will be raised if more than one source is present in the unit and the number of segment/ignorable elements matchin @ref is not 1 -->
            <sch:report test="$source-counter>1 and count(ancestor::xlf:unit//xlf:segment[@id=current()/@ref] | ancestor::xlf:unit//xlf:ignorable[@id=@ref])!=1">
                The corresponding 'segment' or 'ignorable' element with id='<sch:value-of select="@ref"/>', to which 'ref' is pointing was not found within the 'unit'.
            </sch:report>
            <sch:report test="count(child::ctr:revision/ctr:contentItem)!=1">
                When 'appliesTo' is set to 'source' or 'target', the 'revisions' element must have exactly one 'contentItem' grandchild.
            </sch:report>
        </sch:rule>
    </sch:pattern>
    <sch:pattern id="ctr3">
        <!-- Select only those revisions elements which specify @ref-->
        <sch:rule context="ctr:revisions[@ref]">
            <sch:let name="xlf-element" value="@appliesTo"/>
            <sch:let name="ref" value="@ref"/>
            <!-- reference-counter represents the number of xlf element, which have the same name as @appliesTo and the same @id as the @ref of revisions-->
            <sch:let name="reference-counter" value="count(ancestor::xlf:unit//xlf:*[local-name()=$xlf-element][@id=$ref])"/>
            <!-- 'source' and 'target' cases are excluded, but for other elements there must be exactly 1 match-->
            <sch:report test="$xlf-element!='source' and $xlf-element!='target' and $reference-counter!=1">
                The corresponding '<sch:value-of select="$xlf-element"/>' with id='<sch:value-of select="$ref"/>', to which 'ref' is pointing was not found within the 'unit'.
            </sch:report>
        </sch:rule>
    </sch:pattern>
    <sch:pattern id="ctr4">
        <sch:rule context="ctr:revisions[@appliesTo='unit']">
            <sch:report test="count(child::ctr:revision/ctr:unitItem)!=1">
                When 'appliesTo' is set to 'unit', 'revisions' element must have exactly one 'unitItem' grandchild.
            </sch:report>
        </sch:rule>
    </sch:pattern>
    <sch:pattern id="ctr5">
        <sch:rule context="ctr:contentItem">
            <sch:assert test="ancestor::ctr:revisions/@appliesTo='source' or ancestor::ctr:revisions/@appliesTo='taget'">
                'contentItem' can be used only if the grandparent 'revisions' element has set its 'appliesTo' attribute to 'source' or 'target'.
            </sch:assert>
            <sch:assert test="@property='content'">
                Only 'content' is allowed for the value of 'property' attribute when used in 'contentItem'.
            </sch:assert>
        </sch:rule>
    </sch:pattern>
    <sch:pattern id="ctr6">
        <sch:rule context="ctr:unitItem">
            <sch:assert test="ancestor::ctr:revisions/@applieTo='unit'">
                'unitItem' can be used only if the grandparent 'revisions' element has set its 'appliesTo' attribute to 'unit'.
            </sch:assert>
            <sch:assert test="@property='content'">
                Only 'content' is allowed for the value of 'property' attribute when used in'unitItem'.
            </sch:assert>
        </sch:rule>
    </sch:pattern>
    <sch:pattern id="ctr7">
        <sch:rule context="ctr:item[@property='content']">
            <sch:assert test="ancestor::ctr:revisions/@appliesTo='note'">
                The 'content' value for 'property' attribute is allowed only when the enclosing 'revisions' element has its 'appliesTo' attribute set to 'note'.
            </sch:assert>
        </sch:rule>
    </sch:pattern>
    <sch:pattern id="ctr8">
        <sch:rule context="ctr:revision[@version]">
            <sch:assert test="count(ancestor::ctr:revisions//ctr:revision[@version=current()/@version])=1">
                'version' must be unique in the scope 'revisions' element.
            </sch:assert>
        </sch:rule>
    </sch:pattern>
    <sch:pattern id="ctr9">
        <sch:rule context="ctr:contentItem//xlf:sc[@isolated='no' or not(@isolated)]">
            <sch:assert test="ancestor::ctr:contentItem//xlf:ec[@startRef=current()/@id]">
                There must not be any unhandled orphan elements. The 'ec' element corresponding to this start code is missing in the same 'ctr:contentItem'.
            </sch:assert>
        </sch:rule>
    </sch:pattern>
    <sch:pattern id="ctr10">
        <sch:rule context="ctr:contentItem//xlf:sm">
            <sch:assert test="ancestor::ctr:contentItem//xlf:em[@startRef=current()/@id]">
                There must not be any unhandled orphan elements. The 'em' element corresponding to this start marker is missing in the same 'ctr:contentItem'.
            </sch:assert>
        </sch:rule>
    </sch:pattern>
    <sch:pattern id="ctr11">
        <sch:rule context="ctr:segmentItem//xlf:sc[@isolated='no' or not(@isolated)]">
            <sch:assert test="ancestor::ctr:segmentItem//xlf:ec[@startRef=current()/@id]">
                There must not be any unhandled orphan elements. The 'ec' element corresponding to this start code is missing in the same 'ctr:segmentItem'.
            </sch:assert>
        </sch:rule>
    </sch:pattern>
    <sch:pattern id="ctr12">
        <sch:rule context="ctr:segmentItem//xlf:sm">
            <sch:assert test="ancestor::ctr:segmentItem//xlf:em[@startRef=current()/@id]">
                There must not be any unhandled orphan elements. The 'em' element corresponding to this start marker is missing in the same 'ctr:segmentItem'.
            </sch:assert>
        </sch:rule>
    </sch:pattern>
    <sch:pattern id="ctr13">
        <sch:rule context="ctr:unitItem//xlf:sc[@isolated='no' or not(@isolated)]">
           <sch:assert test="ancestor::ctr:unitItem//xlf:ec[@startRef=current()/@id]">
               There must not be any unhandled orphan elements. The 'ec' element corresponding to this start code is missing in the same 'ctr:unitItem'.
           </sch:assert>
       </sch:rule>
    </sch:pattern>
    <sch:pattern id="ctr14">
        <sch:rule context="ctr:unitItem//xlf:sm">
            <sch:assert test="ancestor::ctr:unitItem//xlf:em[@startRef=current()/@id]">
                There must not be any unhandled orphan elements. The 'em' element corresponding to this start marker is missing in the same 'ctr:unitItem'.
            </sch:assert>
        </sch:rule>
    </sch:pattern>
    <sch:diagnostics>
        <sch:diagnostic id="fragid-report">#<sch:value-of select="$fragid"/></sch:diagnostic>
    </sch:diagnostics>
</sch:schema>